﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Web;
using ThinkGeo.MapSuite.Drawing;
using ThinkGeo.MapSuite.Layers;
using ThinkGeo.MapSuite.Shapes;
using ThinkGeo.MapSuite.Styles;

namespace SampleWebApp
{
    class CustomClusterPointStyle : PointStyle
    {
        private double scale2 = 0;
        private int cellSize = 100;
        private int minimumFeaturesPerCellToCluster = 100;
        Collection<Feature> allFeatures = null;
        GeoSolidBrush symbolSolidBrush = null;
        public CustomClusterPointStyle(PointSymbolType symbolType, GeoSolidBrush symbolSolidBrush1, int symbolSize)
            : base(symbolType, symbolSolidBrush1, symbolSize)
        {
            symbolSolidBrush = symbolSolidBrush1;
        }

        public int CellSize
        {
            get { return cellSize; }
            set { cellSize = value; }
        }

        public int MinimumFeaturesPerCellToCluster
        {
            get { return minimumFeaturesPerCellToCluster; }
            set { minimumFeaturesPerCellToCluster = value; }
        }

        public Collection<Feature> AllFeatures
        {
            get { return allFeatures; }
            set { allFeatures = value; }
        }

        protected override void DrawCore(IEnumerable<Feature> features, GeoCanvas canvas, Collection<SimpleCandidate> labelsInThisLayer, Collection<SimpleCandidate> labelsInAllLayers)
        {
            //base.DrawCore(features, canvas, labelsInThisLayer, labelsInAllLayers);

            double scale = ExtentHelper.GetScale(canvas.CurrentWorldExtent, canvas.Width, canvas.MapUnit);
            if (scale2!= scale)
            {
                scale2 = scale;
                MapSuiteTileMatrix mapSuiteTileMatrix = new MapSuiteTileMatrix(scale, cellSize, cellSize, canvas.MapUnit);
                IEnumerable<TileMatrixCell> tileMatricCells = mapSuiteTileMatrix.GetContainedCells(canvas.CurrentWorldExtent);
                Dictionary<string, string> unusedFeatures = new Dictionary<string, string>();
                foreach (Feature feature in allFeatures)
                    unusedFeatures.Add(feature.Id, feature.Id);
                foreach (TileMatrixCell cell in tileMatricCells)
                {
                    int featureCount = 0;
                    MultipointShape multiPointShape = new MultipointShape();
                    foreach (Feature feature in allFeatures)
                    {
                        if (unusedFeatures.ContainsKey(feature.Id))
                        {
                            if (cell.BoundingBox.Contains(feature.GetBoundingBox()))
                            {
                                featureCount++;
                                unusedFeatures.Remove(feature.Id);
                                multiPointShape.Points.Add(feature.GetBoundingBox().GetCenterPoint());
                            }
                        }
                    }
                    if (featureCount > 0)
                    {
                        //Dictionary<string, string> featureValues = new Dictionary<string, string>();
                        //featureValues.Add("FeatureCount", featureCount.ToString());

                        //Draw the point shape  
                        //base.DrawCore(new Feature[] { new Feature(multiPointShape.GetCenterPoint(), featureValues) }, canvas, labelsInThisLayer, labelsInAllLayers);
                        base.DrawCore(new Feature[] { new Feature(multiPointShape.GetCenterPoint()) }, canvas, labelsInThisLayer, labelsInAllLayers);

                        //http://community.thinkgeo.com/t/pass-current-extent-to-draw-method/2411
                        // Draw the text style to show how many feaures are consolidated in the cluster  
                        //textSytle.Draw(new Feature[] { new Feature(multiPointShape.GetCenterPoint(), featureValues) }, canvas, labelsInThisLayer, labelsInAllLayers);
                    }
                }
            }
        }
    }
}